@@ -21,6 +21,7 @@
 #include <grub/normal.h>
 #include <grub/dl.h>
 #include <grub/misc.h>
+#include <grub/term.h>
 #include <grub/ieee1275/ieee1275.h>
 
 static grub_err_t
@@ -30,18 +31,19 @@ grub_cmd_suspend (struct grub_arg_list *state  __attribute__ ((unused)),
 {
   grub_printf ("Run 'go' to resume GRUB.\n");
   grub_ieee1275_enter ();
+  grub_cls ();
   return 0;
 }
 
 
-GRUB_MOD_INIT
+GRUB_MOD_INIT(ieee1275_suspend)
 {
   (void)mod;			/* To stop warning. */
   grub_register_command ("suspend", grub_cmd_suspend, GRUB_COMMAND_FLAG_BOTH,
 			 "suspend", "Return to Open Firmware prompt", 0);
 }
 
-GRUB_MOD_FINI
+GRUB_MOD_FINI(ieee1275_suspend)
 {
   grub_unregister_command ("suspend");
 }
