@@ -1,19 +1,20 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2003,2005,2006,2007,2009  Free Software Foundation, Inc.
+ *  Copyright (C) 2003,2005  Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  GRUB is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with GRUB; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #ifndef GRUB_ENV_HEADER
@@ -22,7 +23,6 @@
 #include <grub/symbol.h>
 #include <grub/err.h>
 #include <grub/types.h>
-#include <grub/menu.h>
 
 struct grub_env_var;
 
@@ -39,31 +39,16 @@ struct grub_env_var
   grub_env_write_hook_t write_hook;
   struct grub_env_var *next;
   struct grub_env_var **prevp;
-  int global;
+  struct grub_env_var *sort_next;
+  struct grub_env_var **sort_prevp;
 };
 
-grub_err_t EXPORT_FUNC(grub_env_set) (const char *name, const char *val);
+grub_err_t EXPORT_FUNC(grub_env_set) (const char *var, const char *val);
 char *EXPORT_FUNC(grub_env_get) (const char *name);
 void EXPORT_FUNC(grub_env_unset) (const char *name);
-void EXPORT_FUNC(grub_env_iterate) (int (*func) (struct grub_env_var *var));
-struct grub_env_var *EXPORT_FUNC(grub_env_find) (const char *name);
-grub_err_t EXPORT_FUNC(grub_register_variable_hook) (const char *name,
+void EXPORT_FUNC(grub_env_iterate) (int (* func) (struct grub_env_var *var));
+grub_err_t EXPORT_FUNC(grub_register_variable_hook) (const char *var,
 						     grub_env_read_hook_t read_hook,
 						     grub_env_write_hook_t write_hook);
 
-grub_err_t grub_env_context_open (void);
-grub_err_t grub_env_context_close (void);
-grub_err_t EXPORT_FUNC(grub_env_export) (const char *name);
-
-void grub_env_unset_menu (void);
-grub_menu_t grub_env_get_menu (void);
-void grub_env_set_menu (grub_menu_t nmenu);
-
-grub_err_t
-grub_env_extractor_open (int source);
-
-grub_err_t
-grub_env_extractor_close (int source);
-
-
 #endif /* ! GRUB_ENV_HEADER */
