@@ -39,7 +39,7 @@ static const grub_addr_t grub_heap_start = 0x4000;
 static grub_addr_t grub_heap_len;
 
 void
-grub_exit (void)
+abort (void)
 {
   /* Trap to Open Firmware.  */
   asm ("trap");
@@ -62,17 +62,13 @@ grub_translate_ieee1275_path (char *filepath)
     }
 }
 
-void
-grub_machine_set_prefix (void)
+static void
+grub_set_prefix (void)
 {
   char bootpath[64]; /* XXX check length */
   char *filename;
   char *prefix;
 
-  if (grub_env_get ("prefix"))
-    /* We already set prefix in grub_machine_init().  */
-    return;
-
   if (grub_ieee1275_get_property (grub_ieee1275_chosen, "bootpath", &bootpath,
 				  sizeof (bootpath), 0))
     {
@@ -116,6 +112,7 @@ void
 grub_machine_init (void)
 {
   char args[256];
+  grub_ieee1275_phandle_t chosen;
   int actual;
   extern char _start;
 
@@ -129,14 +126,17 @@ grub_machine_init (void)
     {
       grub_printf ("Failed to claim heap at 0x%x, len 0x%x\n", grub_heap_start,
 		   grub_heap_len);
-      grub_abort ();
+      abort ();
     }
   grub_mm_init_region ((void *) grub_heap_start, grub_heap_len);
 
+  grub_set_prefix ();
+
   grub_ofdisk_init ();
 
   /* Process commandline.  */
-  if (grub_ieee1275_get_property (grub_ieee1275_chosen, "bootargs", &args,
+  grub_ieee1275_finddevice ("/chosen", &chosen);
+  if (grub_ieee1275_get_property (chosen, "bootargs", &args,
 				  sizeof args, &actual) == 0
       && actual > 1)
     {
@@ -177,6 +177,12 @@ grub_machine_fini (void)
   grub_console_fini ();
 }
 
+void
+grub_stop (void)
+{
+  for (;;);
+}
+
 grub_uint32_t
 grub_get_rtc (void)
 {
