@@ -30,7 +30,7 @@
 #include <grub/machine/memory.h>
 #include <grub/rescue.h>
 #include <grub/dl.h>
-#include <grub/cpu/linux.h>
+#include <grub/machine/linux.h>
 
 static grub_dl_t my_mod;
 
@@ -81,11 +81,9 @@ grub_rescue_cmd_linux (int argc, char *argv[])
   if (! file)
     goto fail;
 
-  if ((grub_size_t) grub_file_size (file) > grub_os_area_size)
+  if (grub_file_size (file) > (grub_ssize_t) grub_os_area_size)
     {
-      grub_error (GRUB_ERR_OUT_OF_RANGE, "too big kernel (0x%x > 0x%x)",
-		  (grub_size_t) grub_file_size (file),
-		  grub_os_area_size);
+      grub_error (GRUB_ERR_OUT_OF_RANGE, "too big kernel");
       goto fail;
     }
 
@@ -275,7 +273,7 @@ grub_rescue_cmd_linux (int argc, char *argv[])
   if (grub_errno == GRUB_ERR_NONE)
     {
       grub_linux_prot_size = prot_size;
-      grub_loader_set (grub_linux_boot, grub_linux_unload, 1);
+      grub_loader_set (grub_linux_boot, grub_linux_unload);
       loaded = 1;
     }
 
@@ -322,19 +320,11 @@ grub_rescue_cmd_initrd (int argc, char *argv[])
 
   /* Get the highest address available for the initrd.  */
   if (grub_le_to_cpu16 (lh->version) >= 0x0203)
-    {
-      addr_max = grub_cpu_to_le32 (lh->initrd_addr_max);
-
-      /* XXX in reality, Linux specifies a bogus value, so
-	 it is necessary to make sure that ADDR_MAX does not exceed
-	 0x3fffffff.  */
-      if (addr_max > GRUB_LINUX_INITRD_MAX_ADDRESS)
-	addr_max = GRUB_LINUX_INITRD_MAX_ADDRESS;
-    }
+    addr_max = grub_cpu_to_le32 (lh->initrd_addr_max);
   else
     addr_max = GRUB_LINUX_INITRD_MAX_ADDRESS;
 
-  if (linux_mem_size != 0 && linux_mem_size < addr_max)
+  if (!linux_mem_size && linux_mem_size < addr_max)
     addr_max = linux_mem_size;
 
   /* Linux 2.3.xx has a bug in the memory range check, so avoid
