@@ -1,20 +1,21 @@
 /* multiboot.c - boot a multiboot OS image. */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2003,2004,2005,2007  Free Software Foundation, Inc.
+ *  Copyright (C) 2003, 2004, 2005  Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 /* 
@@ -32,9 +33,8 @@
 
 #include <grub/loader.h>
 #include <grub/machine/loader.h>
-#include <grub/multiboot.h>
+#include <grub/machine/multiboot.h>
 #include <grub/machine/init.h>
-#include <grub/machine/memory.h>
 #include <grub/elf.h>
 #include <grub/file.h>
 #include <grub/err.h>
@@ -44,7 +44,7 @@
 #include <grub/misc.h>
 #include <grub/gzio.h>
 
-extern grub_dl_t my_mod;
+static grub_dl_t my_mod;
 static struct grub_multiboot_info *mbi;
 static grub_addr_t entry;
 
@@ -108,7 +108,7 @@ grub_multiboot_load_elf32 (grub_file_t file, void *buffer)
     return grub_error (GRUB_ERR_UNKNOWN_OS, "invalid ELF file type");
   
   /* FIXME: Should we support program headers at strange locations?  */
-  if (ehdr->e_phoff + ehdr->e_phnum * ehdr->e_phentsize > MULTIBOOT_SEARCH)
+  if (ehdr->e_phoff + ehdr->e_phnum * ehdr->e_phentsize > GRUB_MB_SEARCH)
     return grub_error (GRUB_ERR_BAD_OS, "program header at a too high offset");
   
   entry = ehdr->e_entry;
@@ -121,18 +121,13 @@ grub_multiboot_load_elf32 (grub_file_t file, void *buffer)
       if (phdr->p_type == PT_LOAD)
         {
           /* The segment should fit in the area reserved for the OS.  */
-          if (phdr->p_paddr < grub_os_area_addr)
+          if ((phdr->p_paddr < grub_os_area_addr)
+              || (phdr->p_paddr + phdr->p_memsz
+		  > grub_os_area_addr + grub_os_area_size))
 	    return grub_error (GRUB_ERR_BAD_OS,
-			       "segment doesn't fit in memory reserved for the OS (0x%lx < 0x%lx)",
-			       phdr->p_paddr, grub_os_area_addr);
-          if (phdr->p_paddr + phdr->p_memsz > grub_os_area_addr + grub_os_area_size)
-	    return grub_error (GRUB_ERR_BAD_OS,
-			       "segment doesn't fit in memory reserved for the OS (0x%lx > 0x%lx)",
-			       phdr->p_paddr + phdr->p_memsz,
-			       grub_os_area_addr + grub_os_area_size);
+			       "segment doesn't fit in memory reserved for the OS");
 
-          if (grub_file_seek (file, (grub_off_t) phdr->p_offset)
-	      == (grub_off_t) -1)
+          if (grub_file_seek (file, phdr->p_offset) == -1)
 	    return grub_error (GRUB_ERR_BAD_OS,
 			       "invalid offset in program header");
 	  
@@ -182,7 +177,7 @@ grub_multiboot_load_elf64 (grub_file_t file, void *buffer)
     return grub_error (GRUB_ERR_UNKNOWN_OS, "invalid ELF file type");
 
   /* FIXME: Should we support program headers at strange locations?  */
-  if (ehdr->e_phoff + ehdr->e_phnum * ehdr->e_phentsize > MULTIBOOT_SEARCH)
+  if (ehdr->e_phoff + ehdr->e_phnum * ehdr->e_phentsize > GRUB_MB_SEARCH)
     return grub_error (GRUB_ERR_BAD_OS, "program header at a too high offset");
 
   /* We still in 32-bit mode */
@@ -199,19 +194,14 @@ grub_multiboot_load_elf64 (grub_file_t file, void *buffer)
       if (phdr->p_type == PT_LOAD)
         {
           /* The segment should fit in the area reserved for the OS.  */
-          if (phdr->p_paddr < (grub_uint64_t) grub_os_area_addr)
+          if ((phdr->p_paddr < (grub_uint64_t) grub_os_area_addr)
+              || (phdr->p_paddr + phdr->p_memsz
+		  > ((grub_uint64_t) grub_os_area_addr
+		     + (grub_uint64_t) grub_os_area_size)))
 	    return grub_error (GRUB_ERR_BAD_OS,
-			       "segment doesn't fit in memory reserved for the OS (0x%lx < 0x%lx)",
-			       phdr->p_paddr, (grub_uint64_t) grub_os_area_addr);
-          if (phdr->p_paddr + phdr->p_memsz
-		  > (grub_uint64_t) grub_os_area_addr + (grub_uint64_t) grub_os_area_size)
-	    return grub_error (GRUB_ERR_BAD_OS,
-			       "segment doesn't fit in memory reserved for the OS (0x%lx > 0x%lx)",
-			       phdr->p_paddr + phdr->p_memsz,
-			       (grub_uint64_t) grub_os_area_addr + (grub_uint64_t) grub_os_area_size);
+			       "segment doesn't fit in memory reserved for the OS");
 	  
-	  if (grub_file_seek (file, (grub_off_t) phdr->p_offset)
-	      == (grub_off_t) -1)
+	  if (grub_file_seek (file, phdr->p_offset) == -1)
 	    return grub_error (GRUB_ERR_BAD_OS,
 			       "invalid offset in program header");
 
@@ -245,14 +235,16 @@ grub_multiboot_load_elf (grub_file_t file, void *buffer)
 }
 
 void
-grub_multiboot (int argc, char *argv[])
+grub_rescue_cmd_multiboot (int argc, char *argv[])
 {
   grub_file_t file = 0;
-  char buffer[MULTIBOOT_SEARCH], *cmdline = 0, *p;
+  char buffer[GRUB_MB_SEARCH], *cmdline = 0, *p;
   struct grub_multiboot_header *header;
   grub_ssize_t len;
   int i;
 
+  grub_dl_ref (my_mod);
+
   grub_loader_unset ();
     
   if (argc == 0)
@@ -268,7 +260,7 @@ grub_multiboot (int argc, char *argv[])
       goto fail;
     }
 
-  len = grub_file_read (file, buffer, MULTIBOOT_SEARCH);
+  len = grub_file_read (file, buffer, GRUB_MB_SEARCH);
   if (len < 32)
     {
       grub_error (GRUB_ERR_BAD_OS, "File too small");
@@ -281,7 +273,7 @@ grub_multiboot (int argc, char *argv[])
        ((char *) header <= buffer + len - 12) || (header = 0);
        header = (struct grub_multiboot_header *) ((char *) header + 4))
     {
-      if (header->magic == MULTIBOOT_MAGIC 
+      if (header->magic == GRUB_MB_MAGIC 
 	  && !(header->magic + header->flags + header->checksum))
 	break;
     }
@@ -292,7 +284,7 @@ grub_multiboot (int argc, char *argv[])
       goto fail;
     }
 
-  if (header->flags & MULTIBOOT_UNSUPPORTED)
+  if (header->flags & GRUB_MB_UNSUPPORTED)
     {
       grub_error (GRUB_ERR_UNKNOWN_OS,
 		  "Unsupported flag: 0x%x", header->flags);
@@ -306,7 +298,7 @@ grub_multiboot (int argc, char *argv[])
   if (! mbi)
     goto fail;
 
-  mbi->flags = MULTIBOOT_INFO_MEMORY;
+  mbi->flags = GRUB_MB_INFO_MEMORY;
 
   /* Convert from bytes to kilobytes.  */
   mbi->mem_lower = grub_lower_mem / 1024;
@@ -328,13 +320,13 @@ grub_multiboot (int argc, char *argv[])
   /* Remove the space after the last word.  */
   *(--p) = '\0';
   
-  mbi->flags |= MULTIBOOT_INFO_CMDLINE;
+  mbi->flags |= GRUB_MB_INFO_CMDLINE;
   mbi->cmdline = (grub_uint32_t) cmdline;
 
-  mbi->flags |= MULTIBOOT_INFO_BOOT_LOADER_NAME;
+  mbi->flags |= GRUB_MB_INFO_BOOT_LOADER_NAME;
   mbi->boot_loader_name = (grub_uint32_t) grub_strdup (PACKAGE_STRING);
 
-  grub_loader_set (grub_multiboot_boot, grub_multiboot_unload, 1);
+  grub_loader_set (grub_multiboot_boot, grub_multiboot_unload);
 
  fail:
   if (file)
@@ -350,7 +342,7 @@ grub_multiboot (int argc, char *argv[])
 
 
 void
-grub_module  (int argc, char *argv[])
+grub_rescue_cmd_module  (int argc, char *argv[])
 {
   grub_file_t file = 0;
   grub_ssize_t size, len = 0;
@@ -375,7 +367,7 @@ grub_module  (int argc, char *argv[])
     goto fail;
 
   size = grub_file_size (file);
-  module = grub_memalign (MULTIBOOT_MOD_ALIGN, size);
+  module = grub_memalign (GRUB_MB_MOD_ALIGN, size);
   if (! module)
     goto fail;
 
@@ -401,7 +393,7 @@ grub_module  (int argc, char *argv[])
   /* Remove the space after the last word.  */
   *(--p) = '\0';
 
-  if (mbi->flags & MULTIBOOT_INFO_MODS)
+  if (mbi->flags & GRUB_MB_INFO_MODS)
     {
       struct grub_mod_list *modlist = (struct grub_mod_list *) mbi->mods_addr;
 
@@ -428,7 +420,7 @@ grub_module  (int argc, char *argv[])
       modlist->pad = 0;
       mbi->mods_count = 1;
       mbi->mods_addr = (grub_uint32_t) modlist;
-      mbi->flags |= MULTIBOOT_INFO_MODS;
+      mbi->flags |= GRUB_MB_INFO_MODS;
     }
 
  fail:
@@ -441,3 +433,19 @@ grub_module  (int argc, char *argv[])
       grub_free (cmdline);
     }
 }
+
+
+GRUB_MOD_INIT(multiboot)
+{
+  grub_rescue_register_command ("multiboot", grub_rescue_cmd_multiboot,
+				"load a multiboot kernel");
+  grub_rescue_register_command ("module", grub_rescue_cmd_module,
+				"load a multiboot module");
+  my_mod = mod;
+}
+
+GRUB_MOD_FINI(multiboot)
+{
+  grub_rescue_unregister_command ("multiboot");
+  grub_rescue_unregister_command ("module");
+}
