@@ -1,7 +1,7 @@
 /* script.c -- Functions to create an in memory description of the script. */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005, 2006  Free Software Foundation, Inc.
+ *  Copyright (C) 2005  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,7 +28,11 @@
    allocations.  The memory is free'ed in case of an error, or
    assigned to the parsed script when parsing was successful.  */
 
-/* XXX */
+/* The memory that was used while parsing and scanning.  */
+static struct grub_script_mem *grub_script_memused;
+
+/* The result of the parser.  */
+struct grub_script_cmd *grub_script_parsed = 0;
 
 /* In case of the normal malloc, some additional bytes are allocated
    for this datastructure.  All reserved memory is stored in a linked
@@ -42,15 +46,15 @@ struct grub_script_mem
 
 /* Return malloc'ed memory and keep track of the allocation.  */
 void *
-grub_script_malloc (struct grub_parser_param *state, grub_size_t size)
+grub_script_malloc (grub_size_t size)
 {
   struct grub_script_mem *mem;
   mem = (struct grub_script_mem *) grub_malloc (size + sizeof (*mem)
 						- sizeof (char));
 
   grub_dprintf ("scripting", "malloc %p\n", mem);
-  mem->next = state->memused;
-  state->memused = mem;
+  mem->next = grub_script_memused;
+  grub_script_memused = mem;
   return (void *) &mem->mem;
 }
 
@@ -72,22 +76,20 @@ grub_script_mem_free (struct grub_script_mem *mem)
 /* Start recording memory usage.  Returns the memory that should be
    restored when calling stop.  */
 struct grub_script_mem *
-grub_script_mem_record (struct grub_parser_param *state)
+grub_script_mem_record (void)
 {
-  struct grub_script_mem *mem = state->memused;
-  state->memused = 0;
-
+  struct grub_script_mem *mem = grub_script_memused;
+  grub_script_memused = 0;
   return mem;
 }
 
 /* Stop recording memory usage.  Restore previous recordings using
    RESTORE.  Return the recorded memory.  */
 struct grub_script_mem *
-grub_script_mem_record_stop (struct grub_parser_param *state,
-			     struct grub_script_mem *restore)
+grub_script_mem_record_stop (struct grub_script_mem *restore)
 {
-  struct grub_script_mem *mem = state->memused;
-  state->memused = restore;
+  struct grub_script_mem *mem = grub_script_memused;
+  grub_script_memused = restore;
   return mem;
 }
 
@@ -106,13 +108,13 @@ grub_script_free (struct grub_script *script)
 /* Extend the argument arg with a variable or string of text.  If ARG
    is zero a new list is created.  */
 struct grub_script_arg *
-grub_script_arg_add (struct grub_parser_param *state, struct grub_script_arg *arg,
+grub_script_arg_add (struct grub_script_arg *arg,
 		     grub_script_arg_type_t type, char *str)
 {
   struct grub_script_arg *argpart;
   struct grub_script_arg *ll;
   
-  argpart = (struct grub_script_arg *) grub_script_malloc (state, sizeof (*arg));
+  argpart = (struct grub_script_arg *) grub_script_malloc (sizeof (*arg));
   argpart->type = type;
   argpart->str = str;
   argpart->next = 0;
@@ -129,15 +131,14 @@ grub_script_arg_add (struct grub_parser_param *state, struct grub_script_arg *ar
 /* Add the argument ARG to the end of the argument list LIST.  If LIST
    is zero, a new list will be created.  */
 struct grub_script_arglist *
-grub_script_add_arglist (struct grub_parser_param *state,
-			 struct grub_script_arglist *list, struct grub_script_arg *arg)
+grub_script_add_arglist (struct grub_script_arglist *list, struct grub_script_arg *arg)
 {
   struct grub_script_arglist *link;
   struct grub_script_arglist *ll;
 
   grub_dprintf ("scripting", "arglist\n");
 
-  link = (struct grub_script_arglist *) grub_script_malloc (state, sizeof (*link));
+  link = (struct grub_script_arglist *) grub_script_malloc (sizeof (*link));
   link->next = 0;
   link->arg = arg;
   link->argcount = 0;
@@ -161,15 +162,15 @@ grub_script_add_arglist (struct grub_parser_param *state,
    contains the name of the command that should be executed.  ARGLIST
    holds all arguments for this command.  */
 struct grub_script_cmd *
-grub_script_create_cmdline (struct grub_parser_param *state,
-			    char *cmdname, struct grub_script_arglist *arglist)
+grub_script_create_cmdline (char *cmdname, struct grub_script_arglist *arglist)
 {
   struct grub_script_cmdline *cmd;
 
   grub_dprintf ("scripting", "cmdline\n");
 
-  cmd = grub_script_malloc (state, sizeof (*cmd));
+  cmd = grub_script_malloc (sizeof (*cmd));
   cmd->cmd.exec = grub_script_execute_cmdline;
+/*   cmd->cmd.free = grub_script_free_cmdline; */
   cmd->cmd.next = 0;
   cmd->arglist = arglist;
   cmd->cmdname = cmdname;
@@ -178,74 +179,33 @@ grub_script_create_cmdline (struct grub_parser_param *state,
 }
 
 /* Create a command that functions as an if statement.  If BOOL is
-   evaluated to true (the value is returned in envvar '?'), the
+   evaluated to true (the value is returned in envvar RESULT), the
    interpreter will run the command TRUE, otherwise the interpreter
    runs the command FALSE.  */
 struct grub_script_cmd *
-grub_script_create_cmdif (struct grub_parser_param *state,
-			  struct grub_script_cmd *exec_to_evaluate,
-			  struct grub_script_cmd *exec_on_true,
-			  struct grub_script_cmd *exec_on_false)
+grub_script_create_cmdif (struct grub_script_cmd *bool,
+			  struct grub_script_cmd *true,
+			  struct grub_script_cmd *false)
 {
   struct grub_script_cmdif *cmd;
 
   grub_dprintf ("scripting", "cmdif\n");
 
-  cmd = grub_script_malloc (state, sizeof (*cmd));
+  cmd = grub_script_malloc (sizeof (*cmd));
   cmd->cmd.exec = grub_script_execute_cmdif;
   cmd->cmd.next = 0;
-  cmd->exec_to_evaluate = exec_to_evaluate;
-  cmd->exec_on_true = exec_on_true;
-  cmd->exec_on_false = exec_on_false;
+  cmd->bool = bool;
+  cmd->true = true;
+  cmd->false = false;
 
   return (struct grub_script_cmd *) cmd;
 }
 
-/* Create a command that adds a menu entry to the menu.  Title is an
-   argument that is parsed to generate a string that can be used as
-   the title.  The sourcecode for this entry is passed in SOURCECODE.
-   The options for this entry are passed in OPTIONS.  */
-struct grub_script_cmd *
-grub_script_create_cmdmenu (struct grub_parser_param *state,
-			    struct grub_script_arg *title,
-			    char *sourcecode,
-			    int options)
-{
-  struct grub_script_cmd_menuentry *cmd;
-  int i;
-
-  /* Skip leading newlines to make the sourcecode better readable when
-     using the editor.  */
-  while (*sourcecode == '\n')
-    sourcecode++;
-
-  /* Having trailing returns can some some annoying conflicts, remove
-     them.  XXX: Can the parser be improved to handle this?  */
-  for (i = grub_strlen (sourcecode) - 1; i > 0; i--)
-    {
-      if (sourcecode[i] != '\n')
-	break;
-      sourcecode[i] = '\0';
-    }
-
-  cmd = grub_script_malloc (state, sizeof (*cmd));
-  cmd->cmd.exec = grub_script_execute_menuentry;
-  cmd->cmd.next = 0;
-  /* XXX: Check if this memory is properly free'ed.  */
-  cmd->sourcecode = sourcecode;
-  cmd->title = title;
-  cmd->options = options;
- 
-  return (struct grub_script_cmd *) cmd;
-}
-
 /* Create a block of commands.  CMD contains the command that should
    be added at the end of CMDBLOCK's list.  If CMDBLOCK is zero, a new
    cmdblock will be created.  */
 struct grub_script_cmd *
-grub_script_add_cmd (struct grub_parser_param *state,
-		     struct grub_script_cmdblock *cmdblock,
-		     struct grub_script_cmd *cmd)
+grub_script_add_cmd (struct grub_script_cmdblock *cmdblock, struct grub_script_cmd *cmd)
 {
   grub_dprintf ("scripting", "cmdblock\n");
 
@@ -254,19 +214,20 @@ grub_script_add_cmd (struct grub_parser_param *state,
 
   if (! cmdblock)
     {
-      cmdblock = (struct grub_script_cmdblock *) grub_script_malloc (state,
-								     sizeof (*cmdblock));
+      cmdblock = (struct grub_script_cmdblock *) grub_script_malloc (sizeof (*cmdblock));
       cmdblock->cmd.exec = grub_script_execute_cmdblock;
       cmdblock->cmd.next = 0;
       cmdblock->cmdlist = cmd;
-      cmd->next = 0;
     }
   else
     {
-      cmd->next = cmdblock->cmdlist;
-      cmdblock->cmdlist = cmd;
+      struct grub_script_cmd **last;
+      for (last = &cmdblock->cmdlist; *last; last = &(*last)->next);
+      *last = cmd;
     }
 
+  cmd->next = 0;
+
   return (struct grub_script_cmd *) cmdblock;
 }
 
@@ -299,51 +260,30 @@ grub_script_parse (char *script, grub_err_t (*getline) (char **))
 {
   struct grub_script *parsed;
   struct grub_script_mem *membackup;
-  struct grub_lexer_param *lexstate;
-  struct grub_parser_param *parsestate;
 
   parsed = grub_malloc (sizeof (*parsed));
   if (! parsed)
     return 0;
 
-  parsestate = grub_malloc (sizeof (*parsestate));
-  if (! parsestate)
-    return 0;
-
-  parsestate->err = 0;
-  parsestate->func_mem = 0;
-  parsestate->memused = 0;
-  parsestate->parsed = 0;
-
   /* Initialize the lexer.  */
-  lexstate = grub_script_lexer_init (script, getline);
-  if (! lexstate)
-    {
-      grub_free (parsed);
-      grub_free (parsestate);
-      return 0;
-    }
+  grub_script_lexer_init (script, getline);
 
-  parsestate->lexerstate = lexstate;
+  grub_script_parsed = 0;
 
-  membackup = grub_script_mem_record (parsestate);
+  membackup = grub_script_mem_record ();
 
-  /* Parse the script.  */
-  if (grub_script_yyparse (parsestate) || parsestate->err)
+  /* Parse the script, the result is stored in
+     `grub_script_parsed'.  */
+  if (grub_script_yyparse ())
     {
       struct grub_script_mem *memfree;
-      memfree = grub_script_mem_record_stop (parsestate, membackup);
+      memfree = grub_script_mem_record_stop (membackup);
       grub_script_mem_free (memfree);
-      grub_free (lexstate);
-      grub_free (parsestate);
       return 0;
     }
 
-  parsed->mem = grub_script_mem_record_stop (parsestate, membackup);
-  parsed->cmd = parsestate->parsed;
-
-  grub_free (lexstate);
-  grub_free (parsestate);
+  parsed->mem = grub_script_mem_record_stop (membackup);
+  parsed->cmd = grub_script_parsed;
 
   return parsed;
 }
