@@ -1,20 +1,21 @@
 /* acorn.c - Read Linux/ADFS partition tables.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005,2007  Free Software Foundation, Inc.
+ *  Copyright (C) 2005  Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include <grub/disk.h>
@@ -27,8 +28,8 @@
 #define LINUX_SWAP_MAGIC   grub_cpu_to_le32 (0xdeafab1e)
 #define LINUX_MAP_ENTRIES  (512 / 12)
 
-#define NONADFS_PARTITION_TYPE_LINUX 9
-#define NONADFS_PARTITION_TYPE_MASK 15
+#define NONADFS_PARTITON_TYPE_LINUX 9
+#define NONADFS_PARTITON_TYPE_MASK 15
 
 struct grub_acorn_boot_block
 {
@@ -50,7 +51,7 @@ static struct grub_partition_map grub_acorn_partition_map;
 
 static grub_err_t
 acorn_partition_map_find (grub_disk_t disk, struct linux_part *m,
-			  grub_disk_addr_t *sector)
+			  unsigned int *sector)
 {
   struct grub_acorn_boot_block boot;
   grub_err_t err;
@@ -60,12 +61,12 @@ acorn_partition_map_find (grub_disk_t disk, struct linux_part *m,
   int i;
 
   err = grub_disk_read (disk, 0xC00 / GRUB_DISK_SECTOR_SIZE, 0,
-			sizeof (struct grub_acorn_boot_block),
-			&boot);
+				   sizeof (struct grub_acorn_boot_block),
+				   (char *)&boot);
   if (err)
     return err;
 
-  if ((boot.flags & NONADFS_PARTITION_TYPE_MASK) != NONADFS_PARTITION_TYPE_LINUX)
+  if ((boot.flags & NONADFS_PARTITON_TYPE_MASK) != NONADFS_PARTITON_TYPE_LINUX)
     goto fail;
 
   for (i = 0; i != 0x1ff; ++i)
@@ -81,11 +82,11 @@ acorn_partition_map_find (grub_disk_t disk, struct linux_part *m,
 
   return grub_disk_read (disk, *sector, 0,
 			 sizeof (struct linux_part) * LINUX_MAP_ENTRIES,
-			 m);
+			 (char *)m);
 
 fail:
   return grub_error (GRUB_ERR_BAD_PART_TABLE,
-		     "Linux/ADFS partition map not found");
+		     "Linux/ADFS partition map not found.");
 
 }
 
@@ -99,7 +100,7 @@ acorn_partition_map_iterate (grub_disk_t disk,
   struct grub_disk raw;
   struct linux_part map[LINUX_MAP_ENTRIES];
   int i;
-  grub_disk_addr_t sector;
+  unsigned int sector;
   grub_err_t err;
 
   /* Enforce raw disk access.  */
@@ -118,7 +119,7 @@ acorn_partition_map_iterate (grub_disk_t disk,
 	  && map[i].magic != LINUX_SWAP_MAGIC)
 	return GRUB_ERR_NONE;
 
-      part.start = sector + map[i].start;
+      part.start = (grub_disk_addr_t)sector + map[i].start;
       part.len = map[i].size;
       part.offset = 6;
       part.index = i;
@@ -136,10 +137,9 @@ acorn_partition_map_probe (grub_disk_t disk, const char *str)
 {
   struct linux_part map[LINUX_MAP_ENTRIES];
   struct grub_disk raw = *disk;
-  unsigned long partnum = grub_strtoul (str, 0, 10) - 1;
-  grub_disk_addr_t sector;
+  unsigned long partnum = grub_strtoul (str, 0, 10);
+  unsigned int sector;
   grub_err_t err;
-  grub_partition_t p;
 
   /* Enforce raw disk access.  */
   raw.partition = 0;
@@ -156,11 +156,11 @@ acorn_partition_map_probe (grub_disk_t disk, const char *str)
       && map[partnum].magic != LINUX_SWAP_MAGIC)
     goto fail;
 
-  p = grub_malloc (sizeof (struct grub_partition));
-  if (! p)
+  grub_partition_t p = grub_malloc (sizeof (struct grub_partition));
+  if (!p)
     return 0;
 
-  p->start = sector + map[partnum].start;
+  p->start = (grub_disk_addr_t)sector + map[partnum].start;
   p->len = map[partnum].size;
   p->offset = 6;
   p->index = partnum;
@@ -175,14 +175,21 @@ fail:
 static char *
 acorn_partition_map_get_name (const grub_partition_t p)
 {
-  return grub_xasprintf ("%d", p->index + 1);
+  char *name;
+
+  name = grub_malloc (13);
+  if (!name)
+    return 0;
+
+  grub_sprintf (name, "%d", p->index);
+  return name;
 }
 
 
 /* Partition map type.  */
 static struct grub_partition_map grub_acorn_partition_map =
 {
-  .name = "part_acorn",
+  .name = "Linux/ADFS partition map",
   .iterate = acorn_partition_map_iterate,
   .probe = acorn_partition_map_probe,
   .get_name = acorn_partition_map_get_name
