@@ -1,7 +1,7 @@
 /* apple.c - Read macintosh partition tables.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2002, 2004  Free Software Foundation, Inc.
+ *  Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -220,27 +220,16 @@ static struct grub_partition_map grub_apple_partition_map =
     .get_name = apple_partition_map_get_name
   };
 
-#ifdef GRUB_UTIL
-void
-grub_apple_partition_map_init (void)
-{
-  grub_partition_map_register (&grub_apple_partition_map);
-}
-
-void
-grub_apple_partition_map_fini (void)
-{
-  grub_partition_map_unregister (&grub_apple_partition_map);
-}
-#else
-GRUB_MOD_INIT
+GRUB_MOD_INIT(apple_partition_map)
 {
   grub_partition_map_register (&grub_apple_partition_map);
+#ifndef GRUB_UTIL
   my_mod = mod;
+#endif
 }
 
-GRUB_MOD_FINI
+GRUB_MOD_FINI(apple_partition_map)
 {
   grub_partition_map_unregister (&grub_apple_partition_map);
 }
-#endif
+
