@@ -1,20 +1,21 @@
 /* grub-setup.c - make GRUB usable */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1999,2000,2001,2002,2003,2004,2005,2006,2007,2008,2010  Free Software Foundation, Inc.
+ *  Copyright (C) 1999,2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
  *
- *  GRUB is free software: you can redistribute it and/or modify
+ *  GRUB is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  GRUB is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with GRUB; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include <config.h>
@@ -25,20 +26,11 @@
 #include <grub/file.h>
 #include <grub/fs.h>
 #include <grub/partition.h>
-#include <grub/msdos_partition.h>
-#include <grub/gpt_partition.h>
+#include <grub/pc_partition.h>
 #include <grub/env.h>
-#include <grub/util/hostdisk.h>
+#include <grub/machine/util/biosdisk.h>
 #include <grub/machine/boot.h>
 #include <grub/machine/kernel.h>
-#include <grub/term.h>
-#include <grub/util/raid.h>
-#include <grub/util/lvm.h>
-#include <grub/util/getroot.h>
-
-static const grub_gpt_part_type_t grub_gpt_partition_type_bios_boot = GRUB_GPT_PARTITION_TYPE_BIOS_BOOT;
-
-#include <grub_setup_init.h>
 
 #include <stdio.h>
 #include <unistd.h>
@@ -47,7 +39,7 @@ static const grub_gpt_part_type_t grub_gpt_partition_type_bios_boot = GRUB_GPT_P
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
-#include <assert.h>
+#include <grub/util/getroot.h>
 
 #define _GNU_SOURCE	1
 #include <getopt.h>
@@ -55,128 +47,110 @@ static const grub_gpt_part_type_t grub_gpt_partition_type_bios_boot = GRUB_GPT_P
 #define DEFAULT_BOOT_FILE	"boot.img"
 #define DEFAULT_CORE_FILE	"core.img"
 
+#ifdef __NetBSD__
+/* NetBSD uses /boot for its boot block.  */
+# define DEFAULT_DIRECTORY	"/grub"
+#else
+# define DEFAULT_DIRECTORY	"/boot/grub"
+#endif
+
+#define DEFAULT_DEVICE_MAP	DEFAULT_DIRECTORY "/device.map"
+
+/* This is the blocklist used in the diskboot image.  */
+struct boot_blocklist
+{
+  grub_uint32_t start;
+  grub_uint16_t len;
+  grub_uint16_t segment;
+} __attribute__ ((packed));
+
 void
 grub_putchar (int c)
 {
   putchar (c);
 }
 
-int
-grub_getkey (void)
-{
-  return -1;
-}
-
-struct grub_handler_class grub_term_input_class;
-struct grub_handler_class grub_term_output_class;
-
 void
 grub_refresh (void)
 {
-  fflush (stdout);
 }
 
 static void
-setup (const char *dir,
+setup (const char *prefix, const char *dir,
        const char *boot_file, const char *core_file,
-       const char *root, const char *dest, int must_embed, int force, int fs_probe)
+       const char *root, const char *dest)
 {
-  char *boot_path, *core_path, *core_path_dev;
+  char *boot_path, *core_path;
   char *boot_img, *core_img;
   size_t boot_size, core_size;
   grub_uint16_t core_sectors;
   grub_device_t root_dev, dest_dev;
-  const char *dest_partmap;
   grub_uint8_t *boot_drive;
-  grub_disk_addr_t *kernel_sector;
+  grub_uint32_t *kernel_sector;
   grub_uint16_t *boot_drive_check;
-  struct grub_boot_blocklist *first_block, *block;
+  struct boot_blocklist *first_block, *block;
   grub_int32_t *install_dos_part, *install_bsd_part;
-  grub_int32_t dos_part, bsd_part;
+  char *install_prefix;
   char *tmp_img;
   int i;
-  grub_disk_addr_t first_sector;
+  unsigned long first_sector;
   grub_uint16_t current_segment
     = GRUB_BOOT_MACHINE_KERNEL_SEG + (GRUB_DISK_SECTOR_SIZE >> 4);
   grub_uint16_t last_length = GRUB_DISK_SECTOR_SIZE;
   grub_file_t file;
   FILE *fp;
-  struct { grub_uint64_t start; grub_uint64_t end; } embed_region;
-  embed_region.start = embed_region.end = ~0UL;
-
-  auto void NESTED_FUNC_ATTR save_first_sector (grub_disk_addr_t sector, unsigned offset,
+  unsigned long first_start = ~0UL;
+  
+  auto void save_first_sector (unsigned long sector, unsigned offset,
 			       unsigned length);
-  auto void NESTED_FUNC_ATTR save_blocklists (grub_disk_addr_t sector, unsigned offset,
+  auto void save_blocklists (unsigned long sector, unsigned offset,
 			     unsigned length);
 
-  auto int NESTED_FUNC_ATTR find_usable_region_msdos (grub_disk_t disk,
-						      const grub_partition_t p);
-  int NESTED_FUNC_ATTR find_usable_region_msdos (grub_disk_t disk __attribute__ ((unused)),
-						 const grub_partition_t p)
+  auto int find_first_partition_start (grub_disk_t disk,
+				       const grub_partition_t p);
+  
+  int find_first_partition_start (grub_disk_t disk __attribute__ ((unused)),
+				  const grub_partition_t p)
     {
-      struct grub_msdos_partition *pcdata = p->data;
-
-      /* There's always an embed region, and it starts right after the MBR.  */
-      embed_region.start = 1;
-
-      /* For its end offset, include as many dummy partitions as we can.  */
-      if (! grub_msdos_partition_is_empty (pcdata->dos_type)
-	  && ! grub_msdos_partition_is_bsd (pcdata->dos_type)
-	  && embed_region.end > p->start)
-	embed_region.end = p->start;
+      struct grub_pc_partition *pcdata = p->data;
 
+      if (! grub_pc_partition_is_empty (pcdata->dos_type)
+	  && ! grub_pc_partition_is_bsd (pcdata->dos_type)
+	  && first_start > p->start)
+	first_start = p->start;
+      
       return 0;
     }
-
-  auto int NESTED_FUNC_ATTR find_usable_region_gpt (grub_disk_t disk,
-						    const grub_partition_t p);
-  int NESTED_FUNC_ATTR find_usable_region_gpt (grub_disk_t disk __attribute__ ((unused)),
-					       const grub_partition_t p)
-    {
-      struct grub_gpt_partentry *gptdata = p->data;
-
-      /* If there's an embed region, it is in a dedicated partition.  */
-      if (! memcmp (&gptdata->type, &grub_gpt_partition_type_bios_boot, 16))
-	{
-	  embed_region.start = p->start;
-	  embed_region.end = p->start + p->len;
-
-	  return 1;
-	}
-
-      return 0;
-    }
-
-  void NESTED_FUNC_ATTR save_first_sector (grub_disk_addr_t sector, unsigned offset,
+  
+  void save_first_sector (unsigned long sector, unsigned offset,
 			  unsigned length)
     {
-      grub_util_info ("the first sector is <%llu,%u,%u>",
+      grub_util_info ("the fist sector is <%lu,%u,%u>",
 		      sector, offset, length);
-
+      
       if (offset != 0 || length != GRUB_DISK_SECTOR_SIZE)
 	grub_util_error ("The first sector of the core file is not sector-aligned");
 
       first_sector = sector;
     }
 
-  void NESTED_FUNC_ATTR save_blocklists (grub_disk_addr_t sector, unsigned offset,
-			unsigned length)
+  void save_blocklists (unsigned long sector, unsigned offset, unsigned length)
     {
-      struct grub_boot_blocklist *prev = block + 1;
+      struct boot_blocklist *prev = block + 1;
 
-      grub_util_info ("saving <%llu,%u,%u> with the segment 0x%x",
+      grub_util_info ("saving <%lu,%u,%u> with the segment 0x%x",
 		      sector, offset, length, (unsigned) current_segment);
-
+      
       if (offset != 0 || last_length != GRUB_DISK_SECTOR_SIZE)
 	grub_util_error ("Non-sector-aligned data is found in the core file");
 
       if (block != first_block
-	  && (grub_le_to_cpu64 (prev->start)
+	  && (grub_le_to_cpu32 (prev->start)
 	      + grub_le_to_cpu16 (prev->len)) == sector)
 	prev->len = grub_cpu_to_le16 (grub_le_to_cpu16 (prev->len) + 1);
       else
 	{
-	  block->start = grub_cpu_to_le64 (sector);
+	  block->start = grub_cpu_to_le32 (sector);
 	  block->len = grub_cpu_to_le16 (1);
 	  block->segment = grub_cpu_to_le16 (current_segment);
 
@@ -184,11 +158,11 @@ setup (const char *dir,
 	  if (block->len)
 	    grub_util_error ("The sectors of the core file are too fragmented");
 	}
-
+      
       last_length = length;
       current_segment += GRUB_DISK_SECTOR_SIZE >> 4;
     }
-
+  
   /* Read the boot image by the OS service.  */
   boot_path = grub_util_get_path (dir, boot_file);
   boot_size = grub_util_get_image_size (boot_path);
@@ -198,13 +172,13 @@ setup (const char *dir,
   boot_img = grub_util_read_image (boot_path);
   free (boot_path);
 
-  /* Set the addresses of variables in the boot image.  */
+  /* Set the addresses of BOOT_DRIVE, KERNEL_SECTOR and BOOT_DRIVE_CHECK.  */
   boot_drive = (grub_uint8_t *) (boot_img + GRUB_BOOT_MACHINE_BOOT_DRIVE);
-  kernel_sector = (grub_disk_addr_t *) (boot_img
+  kernel_sector = (grub_uint32_t *) (boot_img
 				     + GRUB_BOOT_MACHINE_KERNEL_SECTOR);
   boot_drive_check = (grub_uint16_t *) (boot_img
 					+ GRUB_BOOT_MACHINE_DRIVE_CHECK);
-
+  
   core_path = grub_util_get_path (dir, core_file);
   core_size = grub_util_get_image_size (core_path);
   core_sectors = ((core_size + GRUB_DISK_SECTOR_SIZE - 1)
@@ -213,18 +187,21 @@ setup (const char *dir,
     grub_util_error ("The size of `%s' is too small", core_path);
   else if (core_size > 0xFFFF * GRUB_DISK_SECTOR_SIZE)
     grub_util_error ("The size of `%s' is too large", core_path);
-
+  
   core_img = grub_util_read_image (core_path);
+  free (core_path);
 
   /* Have FIRST_BLOCK to point to the first blocklist.  */
-  first_block = (struct grub_boot_blocklist *) (core_img
-						+ GRUB_DISK_SECTOR_SIZE
-						- sizeof (*block));
+  first_block = (struct boot_blocklist *) (core_img
+					   + GRUB_DISK_SECTOR_SIZE
+					   - sizeof (*block));
 
   install_dos_part = (grub_int32_t *) (core_img + GRUB_DISK_SECTOR_SIZE
 				       + GRUB_KERNEL_MACHINE_INSTALL_DOS_PART);
   install_bsd_part = (grub_int32_t *) (core_img + GRUB_DISK_SECTOR_SIZE
 				       + GRUB_KERNEL_MACHINE_INSTALL_BSD_PART);
+  install_prefix = (core_img + GRUB_DISK_SECTOR_SIZE
+		    + GRUB_KERNEL_MACHINE_PREFIX);
 
   /* Open the root device and the destination device.  */
   root_dev = grub_device_open (root);
@@ -244,22 +221,6 @@ setup (const char *dir,
   if (grub_disk_read (dest_dev->disk, 0, 0, GRUB_DISK_SECTOR_SIZE, tmp_img))
     grub_util_error ("%s", grub_errmsg);
 
-  if (dest_dev->disk->partition && fs_probe)
-    {
-      grub_fs_t fs;
-      fs = grub_fs_probe (dest_dev);
-      if (! fs)
-	grub_util_error ("Unable to identify a filesystem in %s; safety check can't be performed.",
-			 dest_dev->disk->name);
-
-      if (! fs->reserved_first_sector)
-	grub_util_error ("%s appears to contain a %s filesystem which isn't known to "
-			 "reserve space for DOS-style boot.  Installing GRUB there could "
-			 "result in FILESYSTEM DESTRUCTION if valuable data is overwritten "
-			 "by grub-setup (--skip-fs-probe disables this "
-			 "check, use at your own risk).", dest_dev->disk->name, fs->name);
-    }
-
   /* Copy the possible DOS BPB.  */
   memcpy (boot_img + GRUB_BOOT_MACHINE_BPB_START,
 	  tmp_img + GRUB_BOOT_MACHINE_BPB_START,
@@ -272,173 +233,100 @@ setup (const char *dir,
 	    GRUB_BOOT_MACHINE_PART_END - GRUB_BOOT_MACHINE_WINDOWS_NT_MAGIC);
 
   free (tmp_img);
-
+  
   /* If DEST_DRIVE is a hard disk, enable the workaround, which is
      for buggy BIOSes which don't pass boot drive correctly. Instead,
      they pass 0x00 or 0x01 even when booted from 0x80.  */
   if (dest_dev->disk->id & 0x80)
     /* Replace the jmp (2 bytes) with double nop's.  */
     *boot_drive_check = 0x9090;
-
-  /* If we hardcoded drive as part of prefix, we don't want to
-     override the current setting.  */
-  if (*install_dos_part != -2)
+  
+  /* If the destination device can have partitions and it is the MBR,
+     try to embed the core image into after the MBR.  */
+  if (dest_dev->disk->has_partitions && ! dest_dev->disk->partition)
     {
-      /* Embed information about the installed location.  */
-      if (root_dev->disk->partition)
+      grub_partition_iterate (dest_dev->disk, find_first_partition_start);
+
+      /* If there is enough space...  */
+      if ((unsigned long) core_sectors + 1 <= first_start)
 	{
-	  if (strcmp (root_dev->disk->partition->partmap->name,
-		      "part_msdos") == 0)
+	  grub_util_info ("will embed the core image into after the MBR");
+	  
+	  /* The first blocklist contains the whole sectors.  */
+	  first_block->start = grub_cpu_to_le32 (2);
+	  first_block->len = grub_cpu_to_le16 (core_sectors - 1);
+	  first_block->segment
+	    = grub_cpu_to_le16 (GRUB_BOOT_MACHINE_KERNEL_SEG
+				+ (GRUB_DISK_SECTOR_SIZE >> 4));
+
+	  /* Make sure that the second blocklist is a terminator.  */
+	  block = first_block - 1;
+	  block->start = 0;
+	  block->len = 0;
+	  block->segment = 0;
+
+	  /* Embed information about the installed location.  */
+	  if (root_dev->disk->partition)
 	    {
-	      struct grub_msdos_partition *pcdata =
+	      struct grub_pc_partition *pcdata =
 		root_dev->disk->partition->data;
-	      dos_part = pcdata->dos_part;
-	      bsd_part = pcdata->bsd_part;
-	    }
-	  else if (strcmp (root_dev->disk->partition->partmap->name,
-			   "part_gpt") == 0)
-	    {
-	      dos_part = root_dev->disk->partition->index;
-	      bsd_part = -1;
+	      
+	      if (strcmp (root_dev->disk->partition->partmap->name,
+			  "pc_partition_map") != 0)
+		grub_util_error ("No PC style partitions found");
+	      
+	      *install_dos_part
+		= grub_cpu_to_le32 (pcdata->dos_part);
+	      *install_bsd_part
+		= grub_cpu_to_le32 (pcdata->bsd_part);
 	    }
 	  else
-	    grub_util_error ("No PC style partitions found");
+	    *install_dos_part = *install_bsd_part = grub_cpu_to_le32 (-1);
+
+	  strcpy (install_prefix, prefix);
+	  
+	  /* Write the core image onto the disk.  */
+	  if (grub_disk_write (dest_dev->disk, 1, 0, core_size, core_img))
+	    grub_util_error ("%s", grub_errmsg);
+
+	  /* The boot image and the core image are on the same drive,
+	     so there is no need to specify the boot drive explicitly.  */
+	  *boot_drive = 0xff;
+	  *kernel_sector = grub_cpu_to_le32 (1);
+
+	  /* Write the boot image onto the disk.  */
+	  if (grub_disk_write (dest_dev->disk, 0, 0, GRUB_DISK_SECTOR_SIZE,
+			       boot_img))
+	    grub_util_error ("%s", grub_errmsg);
+
+	  goto finish;
 	}
-      else
-	dos_part = bsd_part = -1;
     }
-  else
-    {
-      dos_part = grub_le_to_cpu32 (*install_dos_part);
-      bsd_part = grub_le_to_cpu32 (*install_bsd_part);
-    }
-
-  grub_util_info ("dos partition is %d, bsd partition is %d",
-		  dos_part, bsd_part);
-
-  if (! dest_dev->disk->has_partitions)
-    {
-      grub_util_warn ("Attempting to install GRUB to a partitionless disk.  This is a BAD idea.");
-      goto unable_to_embed;
-    }
-
-  if (dest_dev->disk->partition)
-    {
-      grub_util_warn ("Attempting to install GRUB to a partition instead of the MBR.  This is a BAD idea.");
-      goto unable_to_embed;
-    }
-
-  /* Unlike root_dev, with dest_dev we're interested in the partition map even
-     if dest_dev itself is a whole disk.  */
-  auto int NESTED_FUNC_ATTR identify_partmap (grub_disk_t disk,
-					      const grub_partition_t p);
-  int NESTED_FUNC_ATTR identify_partmap (grub_disk_t disk __attribute__ ((unused)),
-					 const grub_partition_t p)
-    {
-      dest_partmap = p->partmap->name;
-      return 1;
-    }
-  dest_partmap = 0;
-  grub_partition_iterate (dest_dev->disk, identify_partmap);
-
-  if (! dest_partmap)
-    {
-      grub_util_warn ("Attempting to install GRUB to a partitionless disk.  This is a BAD idea.");
-      goto unable_to_embed;
-    }
-
-  grub_partition_iterate (dest_dev->disk, (strcmp (dest_partmap, "part_msdos") ?
-					   find_usable_region_gpt : find_usable_region_msdos));
-
-  if (embed_region.end == embed_region.start)
-    {
-      if (! strcmp (dest_partmap, "part_msdos"))
-	grub_util_warn ("This msdos-style partition label has no post-MBR gap; embedding won't be possible!");
-      else
-	grub_util_warn ("This GPT partition label has no BIOS Boot Partition; embedding won't be possible!");
-      goto unable_to_embed;
-    }
-
-  if ((unsigned long) core_sectors > embed_region.end - embed_region.start)
-    {
-      if (core_sectors > 62)
-	grub_util_warn ("Your core.img is unusually large.  It won't fit in the embedding area.");
-      else if (embed_region.end - embed_region.start < 62)
-	grub_util_warn ("Your embedding area is unusually small.  core.img won't fit in it.");
-      else
-	grub_util_warn ("Embedding area is too small for core.img.");
-      goto unable_to_embed;
-    }
-
-
-  grub_util_info ("will embed the core image at sector 0x%llx", embed_region.start);
-
-  *install_dos_part = grub_cpu_to_le32 (dos_part);
-  *install_bsd_part = grub_cpu_to_le32 (bsd_part);
-
-  /* The first blocklist contains the whole sectors.  */
-  first_block->start = grub_cpu_to_le64 (embed_region.start + 1);
-
-  /* These are filled elsewhere.  Verify them just in case.  */
-  assert (first_block->len == grub_host_to_target16 (core_sectors - 1));
-  assert (first_block->segment == grub_host_to_target16 (GRUB_BOOT_MACHINE_KERNEL_SEG
-						    + (GRUB_DISK_SECTOR_SIZE >> 4)));
-
-  /* Make sure that the second blocklist is a terminator.  */
-  block = first_block - 1;
-  block->start = 0;
-  block->len = 0;
-  block->segment = 0;
-
-  /* Write the core image onto the disk.  */
-  if (grub_disk_write (dest_dev->disk, embed_region.start, 0, core_size, core_img))
-    grub_util_error ("%s", grub_errmsg);
-
-  /* FIXME: can this be skipped?  */
-  *boot_drive = 0xFF;
-
-  *kernel_sector = grub_cpu_to_le64 (embed_region.start);
-
-  /* Write the boot image onto the disk.  */
-  if (grub_disk_write (dest_dev->disk, 0, 0, GRUB_DISK_SECTOR_SIZE,
-		       boot_img))
-    grub_util_error ("%s", grub_errmsg);
-
-  goto finish;
-
-unable_to_embed:
-
-  if (must_embed)
-    grub_util_error ("Embedding is not possible, but this is required when "
-		     "the root device is on a RAID array or LVM volume.");
-
-  grub_util_warn ("Embedding is not possible.  GRUB can only be installed in this "
-		  "setup by using blocklists.  However, blocklists are UNRELIABLE and "
-		  "its use is discouraged.");
-  if (! force)
-    grub_util_error ("If you really want blocklists, use --force.");
-
+  
+  /* The core image must be put on a filesystem unfortunately.  */
+  grub_util_info ("will leave the core image on the filesystem");
+  
   /* Make sure that GRUB reads the identical image as the OS.  */
   tmp_img = xmalloc (core_size);
-  core_path_dev = grub_util_get_path (dir, core_file);
-
+  core_path = grub_util_get_path (prefix, core_file);
+  
   /* It is a Good Thing to sync two times.  */
   sync ();
   sync ();
 
 #define MAX_TRIES	5
-
+  
   for (i = 0; i < MAX_TRIES; i++)
     {
       grub_util_info ("attempting to read the core image `%s' from GRUB%s",
-		      core_path_dev, (i == 0) ? "" : " again");
-
+		      core_path, (i == 0) ? "" : " again");
+      
       grub_disk_cache_invalidate_all ();
-
-      file = grub_file_open (core_path_dev);
+      
+      file = grub_file_open (core_path);
       if (file)
 	{
-	  if (grub_file_size (file) != core_size)
+	  if (grub_file_size (file) != (grub_ssize_t) core_size)
 	    grub_util_info ("succeeded in opening the core image but the size is different (%d != %d)",
 			    (int) grub_file_size (file), (int) core_size);
 	  else if (grub_file_read (file, tmp_img, core_size)
@@ -450,7 +338,7 @@ unable_to_embed:
 #if 0
 	      FILE *dump;
 	      FILE *dump2;
-
+	      
 	      dump = fopen ("dump.img", "wb");
 	      if (dump)
 		{
@@ -464,8 +352,8 @@ unable_to_embed:
 		  fwrite (core_img, 1, core_size, dump2);
 		  fclose (dump2);
 		}
-
-#endif
+	      
+#endif      
 	      grub_util_info ("succeeded in opening the core image but the data is different");
 	    }
 	  else
@@ -481,14 +369,14 @@ unable_to_embed:
 
       if (grub_errno)
 	grub_util_info ("error message = %s", grub_errmsg);
-
+      
       grub_errno = GRUB_ERR_NONE;
       sync ();
       sleep (1);
     }
 
   if (i == MAX_TRIES)
-    grub_util_error ("Cannot read `%s' correctly", core_path_dev);
+    grub_util_error ("Cannot read `%s' correctly", core_path);
 
   /* Clean out the blocklists.  */
   block = first_block;
@@ -499,16 +387,16 @@ unable_to_embed:
       block->segment = 0;
 
       block--;
-
+      
       if ((char *) block <= core_img)
 	grub_util_error ("No terminator in the core image");
     }
-
+  
   /* Now read the core image to determine where the sectors are.  */
-  file = grub_file_open (core_path_dev);
+  file = grub_file_open (core_path);
   if (! file)
     grub_util_error ("%s", grub_errmsg);
-
+  
   file->read_hook = save_first_sector;
   if (grub_file_read (file, tmp_img, GRUB_DISK_SECTOR_SIZE)
       != GRUB_DISK_SECTOR_SIZE)
@@ -520,20 +408,44 @@ unable_to_embed:
       != (grub_ssize_t) core_size - GRUB_DISK_SECTOR_SIZE)
     grub_util_error ("Failed to read the rest sectors of the core image");
 
-  grub_file_close (file);
-
-  free (core_path_dev);
+  free (core_path);
   free (tmp_img);
+  
+  *kernel_sector = grub_cpu_to_le32 (first_sector);
 
-  *kernel_sector = grub_cpu_to_le64 (first_sector);
+  /* If the destination device is different from the root device,
+     it is necessary to embed the boot drive explicitly.  */
+  if (root_dev->disk->id != dest_dev->disk->id)
+    *boot_drive = (grub_uint8_t) root_dev->disk->id;
+  else
+    *boot_drive = 0xFF;
 
-  /* FIXME: can this be skipped?  */
-  *boot_drive = 0xFF;
-
-  *install_dos_part = grub_cpu_to_le32 (dos_part);
-  *install_bsd_part = grub_cpu_to_le32 (bsd_part);
+  /* Embed information about the installed location.  */
+  if (root_dev->disk->partition)
+    {
+      struct grub_pc_partition *pcdata =
+	root_dev->disk->partition->data;
 
+      if (strcmp (root_dev->disk->partition->partmap->name,
+		  "pc_partition_map") != 0)
+	grub_util_error ("No PC style partitions found");
+      
+      *install_dos_part
+	= grub_cpu_to_le32 (pcdata->dos_part);
+      *install_bsd_part
+	= grub_cpu_to_le32 (pcdata->bsd_part);
+    }
+  else
+    *install_dos_part = *install_bsd_part = grub_cpu_to_le32 (-1);
+  
+  grub_util_info ("dos partition is %u, bsd partition is %u, prefix is %s",
+		  grub_le_to_cpu32 (*install_dos_part),
+		  grub_le_to_cpu32 (*install_bsd_part),
+		  prefix);
+  strcpy (install_prefix, prefix);
+  
   /* Write the first two sectors of the core image onto the disk.  */
+  core_path = grub_util_get_path (dir, core_file);
   grub_util_info ("opening the core image `%s'", core_path);
   fp = fopen (core_path, "r+b");
   if (! fp)
@@ -541,6 +453,7 @@ unable_to_embed:
 
   grub_util_write_image (core_img, GRUB_DISK_SECTOR_SIZE * 2, fp);
   fclose (fp);
+  free (core_path);
 
   /* Write the boot image onto the disk.  */
   if (grub_disk_write (dest_dev->disk, 0, 0, GRUB_DISK_SECTOR_SIZE, boot_img))
@@ -550,8 +463,7 @@ unable_to_embed:
 
   /* Sync is a Good Thing.  */
   sync ();
-
-  free (core_path);
+  
   free (core_img);
   free (boot_img);
   grub_device_close (dest_dev);
@@ -565,8 +477,6 @@ static struct option options[] =
     {"directory", required_argument, 0, 'd'},
     {"device-map", required_argument, 0, 'm'},
     {"root-device", required_argument, 0, 'r'},
-    {"force", no_argument, 0, 'f'},
-    {"skip-fs-probe", no_argument, 0, 's'},
     {"help", no_argument, 0, 'h'},
     {"version", no_argument, 0, 'V'},
     {"verbose", no_argument, 0, 'v'},
@@ -583,15 +493,13 @@ usage (int status)
 Usage: grub-setup [OPTION]... DEVICE\n\
 \n\
 Set up images to boot from DEVICE.\n\
-DEVICE must be a GRUB device (e.g. ``(hd0,1)'').\n\
+DEVICE must be a GRUB device (e.g. ``(hd0,0)'').\n\
 \n\
   -b, --boot-image=FILE   use FILE as the boot image [default=%s]\n\
   -c, --core-image=FILE   use FILE as the core image [default=%s]\n\
   -d, --directory=DIR     use GRUB files in the directory DIR [default=%s]\n\
   -m, --device-map=FILE   use FILE as the device map [default=%s]\n\
   -r, --root-device=DEV   use DEV as the root device [default=guessed]\n\
-  -f, --force             install even if problems are detected\n\
-  -s, --skip-fs-probe     do not probe for filesystems in DEVICE\n\
   -h, --help              display this message and exit\n\
   -V, --version           print version information and exit\n\
   -v, --verbose           print verbose messages\n\
@@ -608,7 +516,7 @@ static char *
 get_device_name (char *dev)
 {
   size_t len = strlen (dev);
-
+  
   if (dev[0] != '(' || dev[len - 1] != ')')
     return 0;
 
@@ -624,15 +532,15 @@ main (int argc, char *argv[])
   char *dir = 0;
   char *dev_map = 0;
   char *root_dev = 0;
+  char *prefix;
   char *dest_dev;
-  int must_embed = 0, force = 0, fs_probe = 1;
-
+  
   progname = "grub-setup";
 
   /* Check for options.  */
   while (1)
     {
-      int c = getopt_long (argc, argv, "b:c:d:m:r:hVvf", options, 0);
+      int c = getopt_long (argc, argv, "b:c:d:m:r:hVv", options, 0);
 
       if (c == -1)
 	break;
@@ -659,7 +567,7 @@ main (int argc, char *argv[])
 
 	    dir = xstrdup (optarg);
 	    break;
-
+	    
 	  case 'm':
 	    if (dev_map)
 	      free (dev_map);
@@ -673,15 +581,7 @@ main (int argc, char *argv[])
 
 	    root_dev = xstrdup (optarg);
 	    break;
-
-	  case 'f':
-	    force = 1;
-	    break;
-
-	  case 's':
-	    fs_probe = 0;
-	    break;
-
+	    
 	  case 'h':
 	    usage (0);
 	    break;
@@ -700,9 +600,6 @@ main (int argc, char *argv[])
 	  }
     }
 
-  if (verbosity > 1)
-    grub_env_set ("debug", "all");
-
   /* Obtain DEST_DEV.  */
   if (optind >= argc)
     {
@@ -718,15 +615,13 @@ main (int argc, char *argv[])
 
   /* Initialize the emulated biosdisk driver.  */
   grub_util_biosdisk_init (dev_map ? : DEFAULT_DEVICE_MAP);
-
-  /* Initialize all modules. */
-  grub_init_all ();
+  grub_pc_partition_map_init ();
 
   dest_dev = get_device_name (argv[optind]);
   if (! dest_dev)
     {
       /* Possibly, the user specified an OS device file.  */
-      dest_dev = grub_util_get_grub_dev (argv[optind]);
+      dest_dev = grub_util_biosdisk_get_grub_dev (argv[optind]);
       if (! dest_dev)
 	{
 	  fprintf (stderr, "Invalid device `%s'.\n", argv[optind]);
@@ -737,20 +632,30 @@ main (int argc, char *argv[])
     /* For simplicity.  */
     dest_dev = xstrdup (dest_dev);
 
+  prefix = grub_get_prefix (dir ? : DEFAULT_DIRECTORY);
+  
+  /* Initialize filesystems.  */
+  grub_fat_init ();
+  grub_ext2_init ();
+  grub_ufs_init ();
+  grub_minix_init ();
+  grub_hfs_init ();
+  grub_jfs_init ();
+  
   if (root_dev)
     {
       char *tmp = get_device_name (root_dev);
 
       if (! tmp)
 	grub_util_error ("Invalid root device `%s'", root_dev);
-
+      
       tmp = xstrdup (tmp);
       free (root_dev);
       root_dev = tmp;
     }
   else
     {
-      root_dev = grub_util_get_grub_dev (grub_guess_root_device (dir ? : DEFAULT_DIRECTORY));
+      root_dev = grub_guess_root_device (dir ? : DEFAULT_DIRECTORY);
       if (! root_dev)
 	{
 	  grub_util_info ("guessing the root device failed, because of `%s'",
@@ -759,51 +664,31 @@ main (int argc, char *argv[])
 	}
     }
 
-#ifdef __linux__
-  if (grub_util_lvm_isvolume (root_dev))
-    must_embed = 1;
-
-  if (root_dev[0] == 'm' && root_dev[1] == 'd'
-      && root_dev[2] >= '0' && root_dev[2] <= '9')
-    {
-      /* FIXME: we can avoid this on RAID1.  */
-      must_embed = 1;
-    }
-
-  if (dest_dev[0] == 'm' && dest_dev[1] == 'd'
-      && dest_dev[2] >= '0' && dest_dev[2] <= '9')
-    {
-      char **devicelist;
-      int i;
-
-      devicelist = grub_util_raid_getmembers (dest_dev);
-
-      for (i = 0; devicelist[i]; i++)
-	{
-	  setup (dir ? : DEFAULT_DIRECTORY,
-		 boot_file ? : DEFAULT_BOOT_FILE,
-		 core_file ? : DEFAULT_CORE_FILE,
-		 root_dev, grub_util_get_grub_dev (devicelist[i]), 1, force, fs_probe);
-	}
-    }
-  else
-#endif
   /* Do the real work.  */
-    setup (dir ? : DEFAULT_DIRECTORY,
-	   boot_file ? : DEFAULT_BOOT_FILE,
-	   core_file ? : DEFAULT_CORE_FILE,
-	   root_dev, dest_dev, must_embed, force, fs_probe);
+  setup (prefix,
+	 dir ? : DEFAULT_DIRECTORY,
+	 boot_file ? : DEFAULT_BOOT_FILE,
+	 core_file ? : DEFAULT_CORE_FILE,
+	 root_dev, dest_dev);
 
   /* Free resources.  */
-  grub_fini_all ();
+  grub_ext2_fini ();
+  grub_fat_fini ();
+  grub_ufs_fini ();
+  grub_minix_fini ();
+  grub_hfs_fini ();
+  grub_jfs_fini ();
+  
+  grub_pc_partition_map_fini ();
   grub_util_biosdisk_fini ();
-
+  
   free (boot_file);
   free (core_file);
   free (dir);
   free (dev_map);
   free (root_dev);
+  free (prefix);
   free (dest_dev);
-
+  
   return 0;
 }
